@@ -16,7 +16,12 @@ class Template:
         super(Template, cls).__setup__()
         new_sel = ('fifo', 'FIFO')
         if new_sel not in cls.cost_price_method.selection:
-            cls.cost_price_method.selection.append(new_sel)
+            cls.cost_price_method._field.selection.append(new_sel)
+
+
+class Product:
+    __metaclass__ = PoolMeta
+    __name__ = 'product.product'
 
     def get_fifo_move(self, quantity=0.0):
         '''
@@ -29,7 +34,6 @@ class Template:
         Move = pool.get('stock.move')
         Uom = pool.get('product.uom')
         Location = pool.get('stock.location')
-        Template = pool.get('product.template')
 
         locations = Location.search([
             ('type', '=', 'storage'),
@@ -38,15 +42,15 @@ class Template:
         location_ids = [l.id for l in locations]
         with Transaction().set_context(locations=location_ids,
                 stock_date_end=stock_date_end):
-            template = Template(self.id)
+            product = self.__class__(self.id)
         offset = 0
         limit = Transaction().database.IN_MAX
-        avail_qty = template.quantity
+        avail_qty = product.quantity
         fifo_moves = []
 
         while avail_qty > 0.0:
             moves = Move.search([
-                ('product.template.id', '=', template.id),
+                ('product', '=', product.id),
                 ('state', '=', 'done'),
                 ('from_location.type', '=', 'supplier'),
                 ], offset=offset, limit=limit,
@@ -58,7 +62,7 @@ class Template:
             for move in moves:
                 qty = Uom.compute_qty(move.uom,
                         move.quantity - move.fifo_quantity,
-                        template.default_uom, round=False)
+                        product.default_uom, round=False)
                 avail_qty -= qty
 
                 if avail_qty <= quantity:
@@ -73,10 +77,5 @@ class Template:
         fifo_moves.reverse()
         return fifo_moves
 
-
-class Product:
-    __metaclass__ = PoolMeta
-    __name__ = 'product.product'
-
     def recompute_cost_price_fifo(self):
         return self.recompute_cost_price_average()
